/* SI 413 Fall 2011
 * ast.cpp
 * Implementations of (longer) functions in the AST class hierarchy.
 */

#include <cstdlib>
#include <iostream>
using namespace std;

#include "ast.hpp"

int nextLabel = 1;

const char* opstring(Oper op) {
  switch(op) {
    case ADD: return "+";   break;
    case SUB: return "-";   break;
    case MUL: return "*";   break;
    case DIV: return "/";   break;
    case LT:  return "<";   break;
    case GT:  return ">";   break;
    case LE:  return "<=";  break;
    case GE:  return ">=";  break;
    case EQ:  return "=";   break;
    case NE:  return "!=";  break;
    case AND: return "and"; break;
    case OR:  return "or";  break;
    case NOTOP: return "not"; break;
    default: return NULL; // will never reach here.
  }
}

Stmt* Stmt::append(Stmt* a, Stmt* b) {
  if (a->isNull()) return b;
  Stmt* n = a;
  while (!n->next->isNull()) n = n->next;
  n->next = b;
  return a;
}

Stmt::Stmt () {
  static NullStmt nullst;
  next= &nullst;
}

void Id::eval(Frame* f) {
  cerr << "Id::eval not implemented yet." << endl;
}

void Num::eval(Frame* f) {
  cout << "literal " << val << endl;
}

void BoolExp::eval(Frame* f) {
  cerr << "BoolExp::eval not implemented yet." << endl;
}

void ArithOp::eval(Frame* f) {
  args[0]->eval(f);
  args[1]->eval(f);
  switch(op) {
    case ADD: cout << "add" << endl; break;
    case SUB: cout << "sub" << endl; break;
    case MUL: cout << "mul" << endl; break;
    case DIV: cout << "div" << endl; break;
    default: cerr << "bad operation stored; aborting" << endl; exit(3);
  }
}

void CompOp::eval(Frame* f) {
  cerr << "CompOp::eval not implemented yet." << endl;
}

void BoolOp::eval(Frame* f) {
  cerr << "BoolOp::eval not implemented yet." << endl;
}

void NegOp::eval(Frame* f) {
  cerr << "NegOp::eval not implemented yet." << endl;
}

void NotOp::eval(Frame* f) {
  cerr << "NotOp::eval not implemented yet." << endl;
}

void Block::exec(Frame* f) {
  body->execAll(new Frame(f));
}

void IfStmt::exec(Frame* f) {
  int bottom = nextLabel++;
  clause->eval(f);
  cout << "not" << endl
       << "literal L" << bottom << endl
       << "branch" << endl;
  ifblock->exec(f);
  cout << "label " << bottom << endl;
}

void IfElse::exec(Frame* f) {
  cerr << "IfElse::exec not implemented yet." << endl;
}

void WhileStmt::exec(Frame* f) {
  cerr << "IfElse::exec not implemented yet." << endl;
}

void Asn::exec(Frame* f) {
  rhs->eval(f);
  cout << "literal " << f->lookup(lhs->getVal()) << endl
       << "store" << endl;
}

void NewStmt::exec(Frame* f) {
  cerr << "NewStmt::exec not implemented yet." << endl;
}

void Write::exec(Frame* f) {
  val->eval(f);
  cout << "print" << endl;
}

void Read::exec(Frame* f) {
  cerr << "Read::exec not implemented yet." << endl;
}

void Lambda::eval(Frame* f) {
  cerr << "Lambda::eval not implemented yet." << endl;
}

void Funcall::eval(Frame* f) {
  cerr << "Funcall::exec not implemented yet." << endl;
}

void ExpStmt::exec(Frame* f) {
  cerr << "ExpStmt::exec not implemented yet." << endl;
}
